﻿/**
 * Angle Mouse
 *
 *		Jacob O. Wobbrock, Ph.D.
 * 		The Information School
 *		University of Washington
 *		Mary Gates Hall, Box 352840
 *		Seattle, WA 98195-2840
 *		wobbrock@uw.edu
 *		
 * This software is distributed under the "New BSD License" agreement:
 * 
 * Copyright (c) 2009-2011, Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the University of Washington nor the names of its 
 *      contributors may be used to endorse or promote products derived from 
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Jacob O. Wobbrock
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;
using WobbrockLib;

namespace AngleMouse
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application. This version only allows for one
        /// instance of this application to be run at a time. It also attempts to deserialize
        /// any saved preferences for this application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Process thisproc = null;
            Process[] procs = null;
            try
            {
                thisproc = Process.GetCurrentProcess();
                procs = Process.GetProcessesByName(thisproc.ProcessName);
            }
            catch (InvalidOperationException iopex)
            {
                Console.WriteLine(iopex);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (thisproc != null && procs != null)
            {
                if (procs.Length <= 1) // just ourselves so proceed
                {
                    Application.EnableVisualStyles();
                    Application.SetCompatibleTextRenderingDefault(false);

                    // try to deserialize from a saved settings file
                    MainForm frm = MainForm.Deserialize(Application.StartupPath + MainForm.SerializationFile); // read in
                    if (frm == null)
                        frm = new MainForm(); // no *.ini file so create default
                    Application.Run(frm);
                }
                else // app with our name already running
                {
                    int nProcOther = (thisproc.Id == procs[0].Id ? 1 : 0); // which process is the other app?
                    IntPtr hWnd = procs[nProcOther].MainWindowHandle;
                    if (Win32.IsIconic(hWnd)) // is the other instance minimized? 
                        Win32.ShowWindowAsync(hWnd, (int) Win32.SW.RESTORE);
                    Win32.SetForegroundWindow(hWnd);
                }
            }
        }
    }
}
